
local T = {}

-- item lookup tables should be all lowercase!

local ITEMNAMES =
{
    -- TODO: item/ingredient names
    naijacorruptedcostume = " ",
    lisuit = "  ",
    throwbiolight = " ",
    naijasuitpartcape = " ",
    naijasuitpartleg = "  ",
    naijasuitpartarm = "  ",
    naijasuitpartfoot = "  ",
    throwjelly1 = " ",
    throwjelly2 = " ",
    throwjelly3 = " ",
    throwjelly4 = " ",
    throwjelly5 = " ",
    throwjelly6 = " ",
    glowingsoup = " ",

    -- TODO: group names
    --[[perogi = "Perogi",
    meat = "Meat",
    mushroom = "Mushroom",
    oil = "Oil",
    loaf = "Loaf",
    soup = "Soup",]]
}

local ITEMDESC =
{
    weakrainbowsoup = "   ...",
    strongbooze = "",
    testobject = " ",
    
    naijacorruptedcostume = "   ,\n  ...",
    lisuit = "    ",
    throwbiolight = "  ,  ",
    naijasuitpartcape = "    ...\n \n(  )",
    naijasuitpartleg = "",
    naijasuitpartarm = "",
    naijasuitpartfoot = "",
    
    throwjelly1 = " ",
    throwjelly2 = " ",
    throwjelly3 = " ",
    throwjelly4 = " ",
    throwjelly5 = " ",
    throwjelly6 = " ",

}

local ITEMFAIL =
{
    max = "!",
    error = "  --  ?",
    weight = " !",
    fullhealth = "  !",
    dropimportant = " ,  !",
    cookimportant = " ,  !",
    cantcook = "   !",
    maxbox = " !", -- the first person complaining about getting this one gets shot
}



local function lookupNoCase(t, k)
    return rawget(t, k:lower())
end

setmetatable(ITEMNAMES, { __index = lookupNoCase })
setmetatable(ITEMDESC,  { __index = lookupNoCase })

local function multAsPercInv(x)
    local p = 100*(1-x)
    if p < 0 then
        return p .. "%"
    end
    return "+" .. p .. "%"
end

local function multAsPerc(x)
    local p = 100*(x-1)
    if p < 0 then
        return p .. "%"
    end
    return "+" .. p .. "%"
end

local POISON_LEVEL_STR =
{
    [0] = "",
    [1] = "",
    [2] = "",
    [3] = " ",
}

local AURACLEAR =
{
    blind = "  ",
}


--[[local STATS_NAMES =
{
    kills = "Total kills",
}]]

local STRINGBUILDER =
{
    -- auras
    speed         = function(time, val) return " " .. multAsPerc(val) .. "  " .. time .. " " end,
    attackpower   = function(time, val) return "  +" .. val .. "  " .. time .. " " end,
    defense       = function(time, val) return " " .. multAsPercInv(val) .. "  " .. time .. " " end,
    regen         = function(time, val) return " +" .. val .. "  " .. time .. " " end,
    trip          = function()          return "!" end,
    antipoison    = function(time, val)
        local str = "<>"
        if type(val) == "" then
            str = POISON_LEVEL_STR[val] or str
        end
        return " " .. str .. "   ".. time .. " "
    end,
    
    -- other
    heal          = function(val)
        if val < 99 then
            return " +" .. val
        else
            return " "
        end
    end,
    
    addpoison = function() return "" end,
    clearaura = function(val)
        return AURACLEAR[val]
    end,
    throwable = function() return " " end,
    clearpoison = function(val) return "  " .. tostring(POISON_LEVEL_STR[val]) .. "" end,
}

local FLAT_VALUES = { "heal", }

local function buildAuraString(auradata)
    local a, tm, val = unpack(auradata)
    local f = STRINGBUILDER[a]
    if f then
        return f(tm, val)
    else
        return "[:" .. a .. "]: " .. tostring(tm) .. ", " .. tostring(val)
    end
end

local function buildMiscString(data, category)
    local f = STRINGBUILDER[category]
    if f then
        return f(data)
    else
        return "[:" .. tostring(data) .. "]"
    end
end

local function buildSingle(f, to, from, ...)
    if from then
        table.insert(to, f(from, ...))
    end
end

local function buildMultiple(f, to, from, ...)
    if from then
        for _, x in ipairs(from) do
            table.insert(to, f(x, ...))
        end
    end
end

local function buildItemEffectString(itemname)
    local itm = DATA_ITEMS[itemname]
    if not itm then return end
    
    local tab = {}
    for _, valname in ipairs(FLAT_VALUES) do
        local val = itm[valname]
        if val then
            local f = STRINGBUILDER[valname]
            if f then
                table.insert(tab, f(val))
            else
                table.insert(tab, "[" .. valname .. "]: " .. val)
            end
        end
    end
    buildSingle(buildAuraString, tab, itm.aura)
    buildMultiple(buildAuraString, tab, itm.auras)
    buildSingle(buildMiscString, tab, itm.clearaura, " ")
    buildMultiple(buildMiscString, tab, itm.clearauras, " ")
    buildSingle(buildMiscString, tab, itm.poison, "")
    buildSingle(buildMiscString, tab, itm.throwable, "")
    
    return table.concat(tab, "\n")
end

local function _splitCamelCase(c)
    return " " .. c
end

function T.getItemName(s)
    local x = ITEMNAMES[s] or s:gsub("[-]", _splitCamelCase)
    if x:startsWith(" ") then
        x = x:sub(2)
    end
    return x
end

local function getDefaultDesc(s)
    local itm = DATA_ITEMS[s]
    if not itm then return end
    
    if itm.type == "food" then
        return "" --"Cooking ingredient"
    end
end

function T.getItemDesc(s)
    local desc = ITEMDESC[s]
    local eff = buildItemEffectString(s)
    if not desc and (not eff or eff == "") then
        desc = getDefaultDesc(s)
    end
    if desc then
        desc =  desc .. "\n" .. eff
    else
        desc = eff
    end
    if desc == "" then
        desc = nil
    end
    return desc
end

function T.getItemFailureMessage(fail, name, rawname)
    local fmt = ITEMFAIL[fail]
    if fmt then
        return fmt:format(name)
    end
    return "  [" .. tostring(fail) .. "]   " .. name
end

return T

